package serviceController;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.File;
import java.io.IOException;

// This class handles the original crawled data set, and produces the expected data which can be directly used by TwitterClient.
public class DataHandler {
	
	/*
	 * If HandleMark & 0x01 == 0x01: Prepare data set for TwitterClient
	 * If HandleMark & 0x02 == 0x02: Prepare data set for TwitterServer
	 * If HandleMark & 0x03 == 0x03: Build server data from client data
	 * Else: Input Error
	 */
	static void HandleData(int HandleMark){
		
		int NumOfFiles = 0;
		File Folder = null;
		File[] ListOfFolders = null;
		
		try {
			Folder = new File(ServiceController.TwitterClientWorkingDir);
			ListOfFolders = Folder.listFiles();
			
			for (; NumOfFiles < ListOfFolders.length; NumOfFiles ++)
		    	if (ListOfFolders[NumOfFiles].isDirectory()){
		    		if(HandleMark == 0x03)
		    			CopyFolloweesToServerDir(ListOfFolders[NumOfFiles].getName());
		    		if(HandleMark == 0x01)
		    			new File(ListOfFolders[NumOfFiles].getPath() + "/" 
		    				+ ListOfFolders[NumOfFiles].getName() + "_received_updates").delete();
		    	}
		    
			// Delete the obsolete user tweets files and the old server logs
			if(HandleMark == 0x02){
				DeleteOldTweets(ServiceController.TwitterServerWorkingDir);
				DeleteOldLogs(ServiceController.TwitterServerLogDir);
			}
		    
			// Delete the old client logs
			if(HandleMark == 0x01)
				DeleteOldLogs(ServiceController.TwitterClientLogDir);
				
		}catch (Exception e) {
			if( ListOfFolders == null)
				System.out.println("Directory not found: " + ServiceController.TwitterClientWorkingDir);
			else
				e.printStackTrace();
		} 
	}
	
	private static void CopyFolloweesToServerDir(String UID){
		
		BufferedReader br = null;
		BufferedWriter bw = null;
		
		try {
			if(new File(ServiceController.TwitterServerWorkingDir + UID).exists() == false)
				new File(ServiceController.TwitterServerWorkingDir + UID).mkdir();
			
			br = new BufferedReader(new FileReader(ServiceController.TwitterClientWorkingDir + UID + "/" + UID + "_followee"));
			bw = new BufferedWriter(new FileWriter(ServiceController.TwitterServerWorkingDir + UID + "/" + UID + "_followee"));
			String str = null;
				
			while((str = br.readLine()) != null){
				bw.write(str);
				bw.newLine();
				bw.flush();
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			try {
				if(br != null)
					br.close();
				if(bw != null)
					bw.close();
			}catch (Exception e){
				e.printStackTrace();
			}
		}
	}
	
	private static void DeleteOldTweets(String path) throws IOException {  
		File f = new File(path);  
		File[] Folders = f.listFiles();
		
		for(int i = 0; i < Folders.length; i++)
			if(Folders[i].isDirectory()){
				new File(Folders[i].getPath() + "/" + Folders[i].getName() + "_user_tweets").delete();
			}
	}
	
	private static void DeleteOldLogs(String LogDirPath) throws IOException {
		File f = new File(LogDirPath);  
		File[] Files = f.listFiles();
		
		for(int i = 0; i < Files.length; i++){
			if(Files[i].isDirectory()){
				if(Files[i].listFiles().length == 0)
					Files[i].delete();
				else
					DeleteOldLogs(Files[i].getPath());
			}else if(Files[i].isFile())
						Files[i].delete();
		}
	}
}