package toolBox;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.net.BindException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.util.ArrayList;
import java.util.Random;

import serviceController.ServiceController;
import twitterClient.TwitterClient;
import twitterServer.TwitterServer;

public class ConfigManager {
	
	// The path of the configuration file
	private static final String ConfigFilePath = "./config.txt";
	
	// The following three members are only useful for TwitterClient.
	private static int NumberOfServers;
	private static ArrayList<String> ServerIPAddresses;
	public static ArrayList<Integer> ServerPorts; // Also used by 'ServiceController'
	
	static {
		ServerIPAddresses = new ArrayList<String>();
		ServerPorts = new ArrayList<Integer>();
	}
	
	public static int LoadClientConfig(){
		BufferedReader br = null;
		int ClientConfigMark = 0x00;
		
		try{
			br = new BufferedReader(new FileReader(ConfigFilePath)); // This is the configuration file.
			
			String ConfigLine;
			String[] ConfigParameters;
			
			while((ConfigLine = br.readLine()) != null && ClientConfigMark != 0x1F){
				if(ConfigLine.trim().equals("[TwitterClient Configurations]")){
					while((ConfigLine = br.readLine()) != null && ClientConfigMark != 0x1F){
						if(ConfigLine.indexOf("=") == -1)
							continue;
						ConfigParameters = ConfigLine.trim().split("=");
						if(ConfigParameters[0].trim().equals("Client Working Directory")){
							TwitterClient.WorkingDir = ConfigParameters[1].trim();
							ClientConfigMark = ClientConfigMark | 0x01;
						}if(ConfigParameters[0].trim().equals("Client Log Directory")){
							TwitterClient.LogDir = ConfigParameters[1].trim();
							ClientConfigMark = ClientConfigMark | 0x02;
						}else if(ConfigParameters[0].trim().equals("Sleep Interval")){
								TwitterClient.SleepInterval = Long.valueOf(ConfigParameters[1].trim());
								ClientConfigMark = ClientConfigMark | 0x04;
						}else if(ConfigParameters[0].trim().equals("Acceleration Factor")){
								TwitterClient.AccelerationFactor = Long.valueOf(ConfigParameters[1].trim());
								ClientConfigMark = ClientConfigMark | 0x08;
						}
						else if(ConfigParameters[0].trim().equals("Number of Servers")){
								ConfigManager.NumberOfServers = Integer.valueOf(ConfigParameters[1].trim());
								int ServerCount = 0;
								while(true){
									ConfigLine = br.readLine();
									if(ConfigLine.indexOf("=") == -1 || ConfigLine.indexOf("@") == -1 
											|| ConfigLine.contains("#") || ServerCount == ConfigManager.NumberOfServers)
										break;
									ConfigParameters = ConfigLine.trim().split("=");
									ConfigParameters = ConfigParameters[1].trim().split("@"); 
									ServerPorts.add(Integer.valueOf(ConfigParameters[0].trim())); // Get the port
									ServerIPAddresses.add(ConfigParameters[1].trim()); // Get the IP address
									ServerCount++;
								}
								ConfigManager.NumberOfServers = ServerCount;
								ClientConfigMark = ClientConfigMark | 0x10;
						}else
							continue;
					}
				}
			}
			
			if(ClientConfigMark != 0x1F){
				System.out.println("Fatal error: Lack of configurations for TwitterClient in the configuration file.");
				ConfigManager.NumberOfServers = 0;
			}
			
		}catch(Exception e){
			e.printStackTrace();
			ConfigManager.NumberOfServers = 0;
		}finally{
			try {
				if(br != null)
					br.close();
			} catch (final IOException e) {
				e.printStackTrace();
				ConfigManager.NumberOfServers = 0;
			}
		}
		
		return ConfigManager.NumberOfServers;
	}
	
	public static String[] SelectRandomServer(){
		
		Random generator = new Random();
		String[] Server = new String[2];
		Server[0] = Server[1] = null;
		
		if(ConfigManager.NumberOfServers == 0){
			System.out.println("Fatal error: No servers are available.");
			return Server;
		}
		
		int randomIndex = generator.nextInt(ConfigManager.NumberOfServers);
		Server[0] = ServerIPAddresses.get(randomIndex);
		Server[1] = ServerPorts.get(randomIndex).toString();
		
		return Server;
	}
	
	public static ServerSocket[] LoadServerConfig(){
		
		int ServerCount = 0;
		ArrayList<String> IPAddresses = null;
		ArrayList<Integer> Ports = null;
		BufferedReader br = null;
		int ServerConfigMark = 0x00;
		ServerSocket[] TempSock = new ServerSocket[2];
		
		try{
			IPAddresses = new ArrayList<String>();
			Ports = new ArrayList<Integer>();
			br = new BufferedReader(new FileReader(ConfigFilePath)); // This is the configuration file.
			
			String ConfigLine;
			String[] ConfigParameters;
			
			while((ConfigLine = br.readLine()) != null && ServerConfigMark != 0x1F){
				if(ConfigLine.indexOf("=") == -1)
					continue;
				ConfigParameters = ConfigLine.trim().split("=");
				if(ConfigParameters[0].trim().equals("Server Working Directory")){
					TwitterServer.WorkingDir = ConfigParameters[1].trim();
					ServerConfigMark = ServerConfigMark | 0x01;
				}if(ConfigParameters[0].trim().equals("Server Log Directory")){
					TwitterServer.LogDir = ConfigParameters[1].trim();
					ServerConfigMark = ServerConfigMark | 0x02;
				}else if(ConfigParameters[0].trim().equals("Number of Servers")){
					ServerCount = Integer.valueOf(ConfigParameters[1].trim());
					int SvrCnt = 0;
					while(true){
						ConfigLine = br.readLine();
						if(ConfigLine.indexOf("=") == -1 || ConfigLine.indexOf("@") == -1 
								|| ConfigLine.contains("#") || SvrCnt == ServerCount)
							break;
						ConfigParameters = ConfigLine.trim().split("=");
						ConfigParameters = ConfigParameters[1].trim().split("@"); 
						Ports.add(Integer.valueOf(ConfigParameters[0].trim())); // Get the port
						IPAddresses.add(ConfigParameters[1].trim()); // Get the IP address
						SvrCnt++;
					}
					ServerCount = SvrCnt;
					ServerConfigMark = ServerConfigMark | 0x04;
				}else if(ConfigParameters[0].trim().equals("Read Ahead Limit")){
					TwitterServer.ReadAheadLimit= Integer.valueOf(ConfigParameters[1].trim());
					ServerConfigMark = ServerConfigMark | 0x08;
				}else if(ConfigParameters[0].trim().equals("Max Retry Count")){
					TwitterServer.MaxRetryCount= Integer.valueOf(ConfigParameters[1].trim());
					ServerConfigMark = ServerConfigMark | 0x10;
				}else
					continue;
			}
			
			if(ServerConfigMark != 0x1F){
				System.out.println("Fatal error: Lack of configurations for TwitterServer in the configuration file.");
				return null;
			}
			
			for(int i = 0; i < ServerCount; i++){
				try{
					// You can change the length of the waiting queue here, Currently they are set to 1000.
					TempSock[0] = new ServerSocket(Ports.get(i), 1000, InetAddress.getByName(IPAddresses.get(i)));
					TempSock[1] = new ServerSocket(Ports.get(i) + 1, 1000, InetAddress.getByName(IPAddresses.get(i)));
					break;
				}catch(Exception e){
					if(e instanceof BindException){
						TempSock[0] = null;
						TempSock[1] = null;
						continue; // This port has already been binded; another server is also running on this machine.
					}else{
						e.printStackTrace();
						TempSock = null;
					}
				}
			}
			
			if(TempSock[0] == null || TempSock[1] == null ){
				System.out.println("Fatal error: Cannot find a right IP address and a port in the configuration.");
				TempSock = null;
			}
		}catch(Exception e){
			e.printStackTrace();
			TempSock = null;
		}
		
		return TempSock;
	}
	
	public static int LoadServiceControllerConfig(){
		
		String ConfigLine;
		String[] ConfigParameters;
		BufferedReader br = null;
		int ConfigMark = 0x0000;
		int RetValue = 0;
			
		try{
			br = new BufferedReader(new FileReader(ConfigFilePath));
			
			while((ConfigLine = br.readLine()) != null && ConfigMark != 0x01FF){
				if(ConfigLine.indexOf("=") == -1)
					continue;
				ConfigParameters = ConfigLine.trim().split("=");
				if(ConfigParameters[0].trim().equals("Maximum Number of Processes")){
					ServiceController.MaxNumOfProcesses = Integer.valueOf(ConfigParameters[1].trim());
					ConfigMark = ConfigMark | 0x01;
				}else if(ConfigParameters[0].trim().equals("Maximum Number of Threads per Process")){
					ServiceController.MaxNumOfThreads = Integer.valueOf(ConfigParameters[1].trim());
					ConfigMark = ConfigMark | 0x02;
				}else if(ConfigParameters[0].trim().equals("Client Working Directory")){
					ServiceController.TwitterClientWorkingDir = ConfigParameters[1].trim();
					ConfigMark = ConfigMark | 0x04;
				}else if(ConfigParameters[0].trim().equals("Client Log Directory")){
					ServiceController.TwitterClientLogDir = ConfigParameters[1].trim();
					ConfigMark = ConfigMark | 0x08;
				}else if(ConfigParameters[0].trim().equals("Client Path")){
					ServiceController.TwitterClientPath = ConfigParameters[1].trim();
					ConfigMark = ConfigMark | 0x10;
				}else if(ConfigParameters[0].trim().equals("Server Working Directory")){
					ServiceController.TwitterServerWorkingDir = ConfigParameters[1].trim();
					ConfigMark = ConfigMark | 0x20;
				}else if(ConfigParameters[0].trim().equals("Server Log Directory")){
					ServiceController.TwitterServerLogDir = ConfigParameters[1].trim();
					ConfigMark = ConfigMark | 0x40;
				}else if(ConfigParameters[0].trim().equals("Server Path")){
					ServiceController.TwitterServerPath = ConfigParameters[1].trim();
					ConfigMark = ConfigMark | 0x80;
				}else if(ConfigParameters[0].trim().equals("Number of Servers")){
					ServiceController.NumOfServers = Integer.valueOf(ConfigParameters[1].trim());
					int ServerCount = 0;
					while((ConfigLine = br.readLine()) != null){
						if(ConfigLine.indexOf("=") == -1 || ConfigLine.indexOf("@") == -1 
								|| ConfigLine.contains("#") || ServerCount == ServiceController.NumOfServers)
							break;
						ConfigParameters = ConfigLine.trim().split("=");
						ConfigParameters = ConfigParameters[1].trim().split("@"); 
						ServerPorts.add(Integer.valueOf(ConfigParameters[0].trim())); // Get the port
						ServerIPAddresses.add(ConfigParameters[1].trim()); // Get the IP address
						ServerCount++;
					}
					ServiceController.NumOfServers = ServerCount;
					ConfigMark = ConfigMark | 0x0100;
				}else
					continue;
			}
			
			if(ConfigMark != 0x01FF){
				System.out.println("Lack of configurations for TwitterClientController in the configuration file.");
				RetValue = -1; // Initialization failed.
			}
			
		}catch(Exception e){
			e.printStackTrace();
			RetValue = -1;
		}finally{
			try {
				if(br != null)
					br.close();
			} catch (final IOException e) {
				e.printStackTrace();
				RetValue = -1;
			}
		}
		
		return RetValue; 
	}
	
	
	
	// The following code is for testing ConfigManager.
	public static void main(String[] str){
		
		ConfigManager.LoadClientConfig();
		ConfigManager.LoadServerConfig();
		ConfigManager.LoadServiceControllerConfig();
		
		System.out.println();
		System.out.println("TwitterClient.WorkingDir=" + TwitterClient.WorkingDir);
		System.out.println("TwitterClient.LogDir=" + TwitterClient.LogDir);
		System.out.println("TwitterClient.SleepInterval=" + TwitterClient.SleepInterval);
		System.out.println("TwitterClient.AccelerationFactor=" + TwitterClient.AccelerationFactor);
		System.out.println();
		for(int i = 0; i < ConfigManager.NumberOfServers; i++){
			System.out.println("ServerIPAddress=" + ConfigManager.ServerIPAddresses.get(i));
			System.out.println("ServerPort=" + ConfigManager.ServerPorts.get(i));
		}
		System.out.println();
		System.out.println("TwitterServer.WorkingDir=" + TwitterServer.WorkingDir);
		System.out.println("TwitterServer.LogDir=" + TwitterServer.LogDir);
		System.out.println("TwitterServer.ServerPort=" + TwitterServer.ServerPort);
		System.out.println("TwitterServer.ReadAheadLimit=" + TwitterServer.ReadAheadLimit);
		System.out.println("TwitterServer.MaxRetryCount=" + TwitterServer.MaxRetryCount);
		System.out.println();
		System.out.println("ServiceController.MaxNumOfProcesses=" + ServiceController.MaxNumOfProcesses);
		System.out.println("ServiceController.MaxNumOfThreads=" + ServiceController.MaxNumOfThreads);
		System.out.println("ServiceController.TwitterClientWorkingDir=" + ServiceController.TwitterClientWorkingDir);
		System.out.println("ServiceController.TwitterClientLogDir=" + ServiceController.TwitterClientLogDir);
		System.out.println("ServiceController.TwitterClientPath=" + ServiceController.TwitterClientPath);
		System.out.println("ServiceController.TwitterServerWorkingDir=" + ServiceController.TwitterServerWorkingDir);
		System.out.println("ServiceController.TwitterServerLogDir=" + ServiceController.TwitterServerLogDir);
		System.out.println("ServiceController.TwitterServerPath=" + ServiceController.TwitterServerPath);
		System.out.println("ServiceController.NumOfServers=" + ServiceController.NumOfServers);
	}
}
