package twitterClient;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.util.Date;
import java.util.Random;
import java.util.StringTokenizer;

import servermsg.RequestUploadStatus;
import toolBox.ErrorLog;

public class TweetsSender extends Thread {

	private Socket sock;
	private InetAddress ServerAddr;
	private int ServerPort;
	private String uid;
	private int RunningMarkIndex;
	private ErrorLog ErrLog;
	long LastUpdateTime; // It is also used by "CheckUpdates" thread.
	
	
	TweetsSender(InetAddress ServerAddr, int ServerPort, String uid, int RunningMarkIndex){
		this.ServerAddr = ServerAddr;
		this.ServerPort = ServerPort;
		this.uid = uid;
		this.RunningMarkIndex = RunningMarkIndex;
		this.LastUpdateTime = -1; // FIX THIS
		super.setName(uid + "_TweetsSender_Thread");
	}

	public void run(){
		ObjectOutputStream oos = null;
		ObjectInputStream ois = null;
		BufferedReader in = null;
		Date StartTime = null;
		Thread ChkUpdThd = null;
		Random RandomInt = null;
		long WaitTimeSpan = 0L;
		long EstimatedRTT = 0L;
		int ResponseCount = 0;
		int BrokenPipeRetryCnt = 0;
		long OldResponseCount = 0L;
		long OnLineNumInEventFile = 0L;
		final long ReRetryLimit = 500L; 
		
		try{
			ErrLog = new ErrorLog(TwitterClient.LogDir + uid + "_TweetsSender.log");
			ErrLog.logInfo("Thread starts.");
			// TwitterClient.Log(super.getName() + " starts.");
			
			in = new BufferedReader(new FileReader(TwitterClient.WorkingDir + uid + "/" +  uid + "_event"));
			in.mark(1024); // Mark the current position of the stream
			
			StartTime = new Date();
			RandomInt = new Random();
			String OneLine;
			
			while((OneLine = in.readLine()) != null){
				StringTokenizer st = new StringTokenizer(OneLine);
				
				try{
					switch(Integer.valueOf(st.nextToken())){
						case 0:{ // This is an online event
								sleep(Long.valueOf(st.nextToken()) / TwitterClient.AccelerationFactor);
								
								if(BrokenPipeRetryCnt == 0)
									ErrLog.logInfo("Session starts.");
								else
									ErrLog.logInfo("Session restarts: Broken pipe.");
								
								for(int i = 1; ;i++){
									try{
										if(i != 1)
											ErrLog.logInfo("Socket reconnects.");
										sock = new Socket(ServerAddr, ServerPort);
										break;
									} catch(SocketException e){ // Deal with the exception of timeout
										if(e.toString().contains("Connection timed out") && i <= ReRetryLimit){
											WaitTimeSpan = RandomInt.nextInt(10) * EstimatedRTT;
											sleep(WaitTimeSpan);
											continue;
										}else if(i > ReRetryLimit){
											i = 0;
											ErrLog.logInfo("Socket reconnection fails. Wait a long time to restart.");
											WaitTimeSpan = 1000 * 60 * 2 + RandomInt.nextInt(10000) * EstimatedRTT;
											sleep(WaitTimeSpan);
											continue;
										}else
											throw e;
									}
								}
								
								try{
									oos = new ObjectOutputStream(sock.getOutputStream());
									ois = new ObjectInputStream(sock.getInputStream());
								}catch(Exception e){
									if(ois != null)
										ois.close();
									if(oos != null)
										oos.close();
									sock.close();
									in.reset();
									BrokenPipeRetryCnt++;
									if(BrokenPipeRetryCnt <= ReRetryLimit){
										WaitTimeSpan = RandomInt.nextInt(10) * EstimatedRTT;
										sleep(WaitTimeSpan);
										continue;
									}else{
										BrokenPipeRetryCnt = 0;
										ErrLog.logInfo("Session fails. Wait a long time to restart.");
										WaitTimeSpan = 1000 * 60 * 2  + RandomInt.nextInt(10000) * EstimatedRTT;
										sleep(WaitTimeSpan);
										continue;
									}
								}
								
								BrokenPipeRetryCnt = 0;
								TwitterClient.isRunning[RunningMarkIndex] = true;
								
								for(int i = 1; ;i++){
									try{
										if(i != 1)
											ErrLog.logInfo("'UpdatesRequestor' Thread restarts.");
										ChkUpdThd = new UpdatesRequestor(ServerAddr, ServerPort - 1, uid, RunningMarkIndex, this);
										ChkUpdThd.start();
										break;
									}catch(Exception e1){
										if(i <= ReRetryLimit){
											continue;
										}else{
											i = 0;
											ErrLog.logInfo("'UpdatesRequestor' Thread creation fails. Wait a long time to restart.");
											WaitTimeSpan = 1000 * 60 * 2 + RandomInt.nextInt(10000) * EstimatedRTT;
											sleep(WaitTimeSpan);
											continue;
										}
									}
								}
								
								OnLineNumInEventFile++;
								in.mark(1024);
								break;
						}
						case 1:{ // This is an offline event
								sleep(Long.valueOf(st.nextToken()) / TwitterClient.AccelerationFactor);
								
								TwitterClient.isRunning[RunningMarkIndex] = false; // Ask the "CheckUpdates" thread to stop if it is running.
								ChkUpdThd.interrupt(); // Interrupt the "CheckUpdates" thread if it is sleeping.
								ChkUpdThd.join();
								
								try{	
									// Tell the server all statuses have been sent; Use "-1" as the mark of the end.
									oos.writeObject(new RequestUploadStatus(uid, -1, -1, null));
									// FIX THIS: Receive a response from the server. This may NOT be necessary.  
									ois.readObject();
								
									ois.close();
									oos.close();
									sock.close();
								}catch(Exception e){
									ErrLog.logException(e);
									ErrLog.logInfo("Session ends abnormally.");
									in.mark(1024);
									continue;
								}
								
								ois = null;
								oos = null;
								sock = null;
								
								ErrLog.logInfo("Session ends normally.");
								in.mark(1024);
								
								break;
						}
						case 2:{ // upload a tweet to server
								String SleepTime = st.nextToken();
								sleep(Long.valueOf(SleepTime) / TwitterClient.AccelerationFactor);
								long CurrentTime = System.currentTimeMillis();
								long RecvTime = 0L;
								
								try{
									oos.writeObject(new RequestUploadStatus(uid, CurrentTime, -1, OneLine.substring(SleepTime.length() + 3)));	
								}catch(Exception e){
									ErrLog.logInfo("Sends tweet exception. Restart the whole session.");
									if(oos != null)
										oos.close();
									if(ois != null)
										ois.close();
									sock.close();
									in.close();
									
									TwitterClient.isRunning[RunningMarkIndex] = false; // Ask the "CheckUpdates" thread to stop if it is running.
									if(ChkUpdThd.getState() == Thread.State.TIMED_WAITING)
										ChkUpdThd.interrupt(); // Interrupt the "CheckUpdates" thread if it is sleeping.
									ChkUpdThd.join();
									
									in = new BufferedReader(new FileReader(TwitterClient.WorkingDir + uid + "/" +  uid + "_event"));
									in.mark(1024);
									for(int i = 0; (OneLine = in.readLine()) != null ;){
										StringTokenizer tempStr = new StringTokenizer(OneLine);
										if(Integer.valueOf(tempStr.nextToken()) == 0){ // This is an online event.
											i++;
											if(i == OnLineNumInEventFile){
												in.reset();
												break;
											}
										}
										in.mark(1024);
									}
							
									continue;
								}
								
								try{
									ErrLog.logInfo("Sends tweet: " + CurrentTime + " " + OneLine.substring(SleepTime.length() + 3));
									
									// FIX THIS: Receive a response from the server. This may NOT be necessary. 
									ois.readObject();
									
									RecvTime = System.currentTimeMillis();
									OldResponseCount = ResponseCount++;
									EstimatedRTT = (long)((EstimatedRTT * OldResponseCount + (RecvTime - CurrentTime)) * 1.0 / ResponseCount);
									ErrLog.logInfo("Receives ack: RTT: " + Long.toString(RecvTime - CurrentTime) + " ms");
								}finally{
									in.mark(1024);
								}
		
								break;
						}
					}
				}catch(Exception e){
					if(ErrLog != null){
						ErrLog.logException(e);
						ErrLog.logInfo("Thread ends abnormally.");
						TwitterClient.Log(super.getName() + " has a session exception.");
					}
					else
						e.printStackTrace();
					
					break;
				}
			}
			
			if(OneLine == null)
				ErrLog.logInfo("Thread ends normally.");
			// TwitterClient.Log(super.getName() + " ends normally.");
			
		}catch(Exception e){
			if(ErrLog != null){
				ErrLog.logException(e);
				ErrLog.logInfo("Thread ends abnormally.");
				TwitterClient.Log(super.getName() + " ends abnormally.");
			}
			else
				e.printStackTrace();
		}finally{
			try{
				if(oos != null && sock.isConnected())
					oos.close();
				if(ois != null && sock.isConnected())
					ois.close();
				if(in != null)
					in.close();
				if(sock != null)
					sock.close();
			}catch(Exception e){
				if(ErrLog != null)
					ErrLog.logException(e);
				else
					e.printStackTrace();
			} finally {
				try{
					if(ErrLog != null){
					// Calculate the time this thread spends, measured in milliseconds.
					ErrLog.logInfo("Thread summary: Starts: " + StartTime.toString() 
							+ " Ends: " + new Date().toString()
							+ " Duration: " + Long.toString((new Date().getTime() - StartTime.getTime()))
							+ " ms");
					ErrLog.closeLog();
					}
				}catch(Exception e1){
					e1.printStackTrace();
				}
			}
		}
	}
}