package twitterClient;

import java.io.BufferedWriter;
import java.io.File;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Date;
import java.io.FileWriter;
import java.io.IOException;

import servermsg.RequestRecentUpdates;
import servermsg.ResponseRecentUpdates;
import toolBox.ErrorLog;


public class UpdatesRequestor extends Thread {
	
	private Socket sock;
	private InetAddress ServerAddr;
	private int ServerPort;
	private String uid;
	private int RunningMarkIndex;
	private TweetsSender UplStsThd;
	private ErrorLog ErrLog;
	// private int NumOfRequests; // We do not need this when writing updates from the server to the log file.
	
	UpdatesRequestor(InetAddress ServerAddr, int ServerPort, String uid, int RunningMarkIndex, TweetsSender Thd){
		this.ServerAddr = ServerAddr;
		this.ServerPort = ServerPort;
		this.uid = uid;
		this.RunningMarkIndex = RunningMarkIndex;
		this.UplStsThd = Thd;
		super.setName(uid + "_UpdatesRequestor_Thread");
	}
	
	public void run(){
		ObjectOutputStream oos = null;
		ObjectInputStream ois = null;
		Date StartTime = null;
		long TempTime = 0;
		
		try{
			StartTime = new Date();
			
			ErrLog = new ErrorLog(TwitterClient.LogDir + uid + "_UpdatesRequestor.log");
			ErrLog.logInfo("Thread starts.");
			// TwitterClient.Log(super.getName() + " starts.");
			
			sock = new Socket(ServerAddr, ServerPort);
			oos = new ObjectOutputStream(sock.getOutputStream());
			ois = new ObjectInputStream(sock.getInputStream());
		
			while(TwitterClient.isRunning[RunningMarkIndex] == true){ //This loop will never stop unless asked by the "UploadStatus" thread.
				try{ 
					sleep(TwitterClient.SleepInterval); // The sleep can be interrupted by the "UploadStatus" thread
				}  
				catch(InterruptedException e){ 
					break;
				} 
				
				// Ask for all updates
				TempTime = System.currentTimeMillis();
				oos.writeObject(new RequestRecentUpdates(uid, TempTime, -1, 1, null, UplStsThd.LastUpdateTime));
				oos.flush();
				
				ErrLog.logInfo("Asks for updates.");
				
				// Read updates from the server
				ResponseRecentUpdates rru = (ResponseRecentUpdates)(ois.readObject());
				WriteUpdates(rru, TempTime);
				
				UplStsThd.LastUpdateTime = TempTime;
			}
			
			ErrLog.logInfo("Thread ends normally.");
			// TwitterClient.Log(super.getName() + " ends normally.");
			
		}catch(Exception e){
			if(ErrLog != null){
				ErrLog.logException(e);
				ErrLog.logInfo("Thread ends abnormally.");
				TwitterClient.Log(super.getName() + " ends abnormally.");
			}else
				e.printStackTrace();
		}finally{
			try{
				if(oos != null && sock.isConnected())
					oos.close();
				if(ois != null && sock.isConnected())
					ois.close();
				if(sock != null)
					sock.close();
			}catch(Exception e){
				if(ErrLog != null)
					ErrLog.logException(e);
				else
					e.printStackTrace();
			} finally {
				try{
					if(ErrLog != null){
						ErrLog.logInfo("Thread summary: Starts: " + StartTime.toString() 
								+ " Ends: " + new Date().toString()
								+ " Duration: " + Long.toString((new Date().getTime() - StartTime.getTime()))
								+ " ms");
						ErrLog.closeLog();
					}
				}catch(Exception e1){
					e1.printStackTrace();
				}
			}
		}
	}
	
	private void WriteUpdates(ResponseRecentUpdates rru, long SendTime){
		
		BufferedWriter bw = null;
		try {
			/*
			 * We used to write updates from the server to the _received_updates file.
			 * But later we found it was redundant thus we wrote these updates directly
			 * to the log file. Notice that we still keep the the _received_updates file.
			 */
			File NewUpdatesFile = new File(TwitterClient.WorkingDir + rru.getUserId() + "/" + rru.getUserId() + "_received_updates");
			if(NewUpdatesFile.exists() == false)
				NewUpdatesFile.createNewFile();
			bw = new BufferedWriter(new FileWriter(NewUpdatesFile, true));
			/*
			bw.write("Request " + Long.toString(++NumOfRequests) + ":");
			bw.newLine();
			bw.flush();
			*/
			
			ErrLog.logInfo("Receives " + rru.getStatuses().size() + " update(s): RTT: " + Long.toString(System.currentTimeMillis() - SendTime)+ " ms");
			for(int i = 0; i < rru.getStatuses().size(); i++){
				/*
				bw.write("FolloweeID: " + (rru.getStatuses().get(i)).getAuthorId() + " " +
						 	"CreatedAt: " + Long.toString((rru.getStatuses().get(i)).getTime()) + " " + 
						 	"Tweet: " + (rru.getStatuses().get(i)).getText());
				bw.newLine();
				bw.flush();
				*/
				ErrLog.logInfo((rru.getStatuses().get(i)).getAuthorId() + " "
						+ Long.toString((rru.getStatuses().get(i)).getTime()) + " "
						+ (rru.getStatuses().get(i)).getText());
			}
		} catch (IOException e) {
			if(ErrLog != null)
				ErrLog.logException(e);
			else
				e.printStackTrace();
		} finally {
			if(bw != null)
				try {
					bw.close();
				} catch (Exception e) {
					if(ErrLog != null)
						ErrLog.logException(e);
					else
						e.printStackTrace();
				}
		}
	}
	

}
