package twitterServer;

import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileWriter;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.SocketException;
import java.util.Date;

import servermsg.RequestUploadStatus;
import servermsg.ResponseAcknowledgement;
import toolBox.ErrorLog;

public class TweetsReceiver implements Runnable{
	private Socket sock;
	private ErrorLog ErrLog;
	
	TweetsReceiver(Socket sock){
		TwitterServer.AddThread();
		this.sock = sock;
		ErrLog = null;
	}
	
	public void run(){
		ObjectOutputStream oos = null;
		ObjectInputStream ois = null;
		BufferedWriter bw = null;
		Date StartTime = null;
		String UId = null;
		long ReceiveTime = -1;
		
		try{
			oos = new ObjectOutputStream(sock.getOutputStream());
			ois = new ObjectInputStream(sock.getInputStream());
				
			while(true){
				RequestUploadStatus rus = (RequestUploadStatus)(ois.readObject());
				ReceiveTime = System.currentTimeMillis();
				
				if(rus.getSendTime() == -1){ // The Client has finished sending statues
					if(bw != null){
						bw.close();
						bw = null;
					}
					/* 
					 * Send this response to tell the client that we have already closed the user tweet files,
					 * thus the client can start the next session.
					 */
					oos.writeObject(new ResponseAcknowledgement(rus.getUserId(), System.currentTimeMillis(), ReceiveTime, rus.getText()));
					if(ErrLog != null)
						ErrLog.logInfo("Thread ends normally.");
					return;
				}
				
				// Open the file and write the status;
				if(bw == null){
					UId = rus.getUserId();
					Thread.currentThread().setName(UId + "_TweetsReceiver_Thread");
					File NewTweetsFile = new File(TwitterServer.WorkingDir + UId + "/" + UId + "_user_tweets");
					if(NewTweetsFile.exists() == false)
						NewTweetsFile.createNewFile();	
					bw = new BufferedWriter(new FileWriter(NewTweetsFile, true));
					ErrLog = new ErrorLog(TwitterServer.LogDir + UId + "_TweetsReceiver.log");
					ErrLog.logInfo("Thread starts.");
					StartTime = new Date();
				}
				
				bw.write(Long.toString(rus.getSendTime()) + " " + rus.getText());
				bw.newLine();
				bw.flush();
				
				ErrLog.logInfo("Receives tweet: " + Long.toString(rus.getSendTime()) + " " + rus.getText());
				
				// return a response to the client
				oos.writeObject(new ResponseAcknowledgement(rus.getUserId(), rus.getSendTime(), ReceiveTime, rus.getText()));	
				ErrLog.logInfo("Sends the acknowledgement.");
			}
		}catch(Exception e){
			if(!(e instanceof SocketException) && !(e instanceof EOFException)){
				if(ErrLog != null){
					ErrLog.logException(e);
					ErrLog.logInfo("Thread ends abnormally.");
				}else
					e.printStackTrace();
			}else if(ErrLog != null) // FIX THIS: The client uses a rude way to exit; this MAY potentially cause problems.
						ErrLog.logInfo("Thread ends normally (Expected exception).");
		}finally{
			try{
				TwitterServer.RemoveThread();
				if(bw != null)
					bw.close();
				if(oos != null && sock.isConnected())
					oos.close();
				if(ois != null && sock.isConnected())
					ois.close();
				if(sock != null)
					sock.close();
			}catch(Exception e){
				if(ErrLog != null)
					ErrLog.logException(e);
				else
					e.printStackTrace();
			}finally{
				try{
					if(ErrLog != null){
						ErrLog.logInfo("Thread summary: Starts: " + StartTime.toString() 
								+ " Ends: " + new Date().toString()
								+ " Duration: " + Long.toString((new Date().getTime() - StartTime.getTime()))
								+ " ms");
						ErrLog.closeLog();
					}
				}catch(Exception e1){
					e1.printStackTrace();
				}
			}
		}
	}
}
