package twitterServer;

import java.io.BufferedReader;
import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.StringTokenizer;

import application.Status;

import servermsg.RequestRecentUpdates;
import servermsg.ResponseRecentUpdates;
import toolBox.ErrorLog;


public class UpdatesChecker implements Runnable{
	private Socket sock;
	private ArrayList<String> Followees;
	private BufferedReader[] TweetsFileReaders;
	private boolean isResponseReady;
	private ErrorLog ErrLog;
	
	UpdatesChecker(Socket sock){
		this.sock = sock;
		Followees = new ArrayList<String>();
		TweetsFileReaders = null;
		isResponseReady = false;
		ErrLog = null;
	}
	
	public void run(){
		ObjectOutputStream oos = null; 
		ObjectInputStream ois = null;
		Date StartTime = null;
		String UId = null;
		long RecvTime = -1;
		
		try{			
			oos = new ObjectOutputStream(sock.getOutputStream());
			ois = new ObjectInputStream(sock.getInputStream());

			while(true){
				RequestRecentUpdates rru = (RequestRecentUpdates)(ois.readObject());
				RecvTime = System.currentTimeMillis();
				
				if(rru.getMark() != 1) {// Currently we deal with ONLY one kind of the requests
					if(ErrLog != null)
						ErrLog.logError("Cannot deal with other kinds of messages. Ends abnormally.");
			
					return;
				}
					
				if(isResponseReady == false){
					UId = rru.getUserId();
					MakeResponseReady(UId);
					isResponseReady = true;
					StartTime = new Date();
				}
				
				ReturnAllUpdates(rru, oos, RecvTime);
			}
		}catch(Exception e){
			if(!(e instanceof SocketException) && !(e instanceof EOFException)){
				if(ErrLog != null){
					ErrLog.logException(e);
					ErrLog.logInfo("Thread ends abnormally.");
				}else
					e.printStackTrace();
			}else if(ErrLog != null)
						ErrLog.logInfo("Thread ends normally.");
		}finally{
			try {
				if(TweetsFileReaders != null){
					for(int i = 0; i < TweetsFileReaders.length; i++)
						if(TweetsFileReaders[i] != null)
							TweetsFileReaders[i].close();
				}
				if(oos != null && sock.isConnected())
					oos.close();
				if(ois != null && sock.isConnected())
					ois.close();
				if(sock != null)
					sock.close();
			} catch (Exception e) {
				if(ErrLog != null)
					ErrLog.logException(e);
				else
					e.printStackTrace();
			} finally {
				try{
					if(ErrLog != null){
						ErrLog.logInfo("Thread summary: Starts: " + StartTime.toString() 
								+ " Ends: " + new Date().toString()
								+ " Duration: " + Long.toString((new Date().getTime() - StartTime.getTime()))
								+ " ms");
						ErrLog.closeLog();
					}
				}catch(Exception e1){
					e1.printStackTrace();
				}
			}
		}
	}
	
	private void MakeResponseReady(String UId){
		
		BufferedReader br = null;
		String str = null;
		
		try {
			br = new BufferedReader(new FileReader(TwitterServer.WorkingDir + UId + "/" + UId + "_followee"));
			while((str = br.readLine()) !=  null)
				Followees.add(str); // Read all followees
			
			TweetsFileReaders = new BufferedReader[Followees.size()];
			for (int i = 0; i < Followees.size(); i++)
				TweetsFileReaders[i] = null;
			
			Thread.currentThread().setName(UId + "_UpdatesChecker_Thread");
			ErrLog = new ErrorLog(TwitterServer.LogDir + UId + "_UpdatesChecker.log");
			ErrLog.logInfo("Thread starts.");
		} catch (Exception e) {
			if(!(e instanceof FileNotFoundException))
				if(ErrLog != null)
					ErrLog.logException(e);
				else
					e.printStackTrace();
		} finally {
			try {
				if(br != null)
					br.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	// return all followees' all updates until currently.
	private void ReturnAllUpdates(RequestRecentUpdates rru, ObjectOutputStream oos, long RecvTime){
		String FolloweeID = null, TempStr = null;
		String[] SplittedString = null;
		
		try{
			// get all the UserIDs of all followees	
			LinkedList<Status> Tweets = new LinkedList<Status>();
			BufferedReader brTweets = null;
			
			ErrLog.logInfo("Sends updates.");
			
			for(int index = 0; index < Followees.size(); index++){
				FolloweeID = Followees.get(index);
				
				if(TweetsFileReaders[index] == null){
					try{
						brTweets = new BufferedReader(new FileReader(TwitterServer.WorkingDir + FolloweeID + "/" + FolloweeID + "_user_tweets"));
						TweetsFileReaders[index] = brTweets;
					}catch(FileNotFoundException e){
						ErrLog.logInfo("No updates: " + FolloweeID + "_user_tweets has not been created yet.");
						continue; //This followee hasn't uploaded any status to the server.
					}catch(Exception e){
						ErrLog.logException(e);
						continue;
					}
				}
			
				SplittedString = new String[2];
				TweetsFileReaders[index].mark(TwitterServer.ReadAheadLimit);
				
				int RetryCount = 0;
				
				while((TempStr = TweetsFileReaders[index].readLine()) != null){
					
					if(TempStr.endsWith("\0") == false){ // This piece of tweet has not been written completely by another server thread.
						TweetsFileReaders[index].reset();
						RetryCount++;
						if(RetryCount > TwitterServer.MaxRetryCount){
							ErrLog.logError("**" + FolloweeID + "_user_tweets has incomplete tweets**: All " + TwitterServer.MaxRetryCount + " read attempts failed.");
							break;
						}
						continue;
					}
					
					RetryCount = 0;
					
					StringTokenizer st = new StringTokenizer(TempStr);
					
					SplittedString[0] = st.nextToken(); // The real-world time when this tweet is created at client
					SplittedString[1] = TempStr.substring(SplittedString[0].length() + 1); // The content of this tweet
					
					if(Long.valueOf(SplittedString[0]) <= rru.getUpdateTime()){
						TweetsFileReaders[index].mark(TwitterServer.ReadAheadLimit);
						continue;
					}else if(Long.valueOf(SplittedString[0]) <= rru.getSendTime()){
						Status OneStatus = new Status(null, SplittedString[1], null, FolloweeID, Long.valueOf(SplittedString[0]));
						Tweets.add(OneStatus);
						ErrLog.logInfo("Sends updates: " + FolloweeID + " " + TempStr);
						TweetsFileReaders[index].mark(TwitterServer.ReadAheadLimit);
					}else {
						ErrLog.logInfo("**A normal reset operation happens here**: at " + FolloweeID + "_user_tweets");
						TweetsFileReaders[index].reset();
						break;
					}
				}
			}
						
			ResponseRecentUpdates response = new ResponseRecentUpdates(rru.getUserId(), System.currentTimeMillis(), RecvTime, Tweets);
			oos.writeObject(response);
			oos.flush();
			
			ErrLog.logInfo("Sent.");
			
		}catch(Exception e){
			if(ErrLog != null)
				ErrLog.logException(e);
			else
				e.printStackTrace();
		}
	}
}
